local moduleName = "Basics"
local core = SimpleUnitFrames

local Basics = core:NewModule(moduleName, "AceEvent-2.0")
local DEFAULT_FONT = "GameFontHighlightSmall"
local DEFAULT_FONT_SIZE = 10
local creatureTypeMap = core.loc.UnitCreatureType

-- List of fonts to be shown in the configuration.
local fonts = {
	"GameFontHighlight",
	"GameFontHighlightSmall",
	"GameFontNormal",
	"GameFontNormalSmall",
	"TextStatusBarText",
}

local inVehicle = {}
local fsParent = {}

-- Static FontString display styles
local staticStyles = {
	-- Health Styles
	["HPcurrent"] = '[HP]',
	["HPdeficit"] = '[0-MissingHP]',
	["HPpercent"] = '[PercentHP:Percent]',
	["HPcurrmax"] = '[FractionalHP:Short]',
	["HPcomplete"] = '[FractionalHP:Short] [PercentHP:Percent:Paren]',
	-- Mana Styles
	["MPcurrent"] = '[HasMP ? MP ! "-"]',
	["MPdeficit"] = '[HasMP ? 0-MissingMP]',
	["MPpercent"] = '[HasMP ? PercentMP:Percent ! "-"]',
	["MPcurrmax"] = '[HasMP ? FractionalMP:Short ! "-"]',
	["MPcomplete"] = '[HasMP ? FractionalMP:Short " " PercentMP:Percent:Paren ! "-"]',
	-- No Style
	["none"] = '',
}

local unitMap = {}

local icons = {
	DEATHKNIGHT = "DEATHKNIGHT",
	DRUID = "DRUID",
	HUNTER = "HUNTER",
	MAGE = "MAGE",
	PALADIN = "PALADIN",
	PRIEST = "PRIEST",
	ROGUE = "ROGUE",
	SHAMAN = "SHAMAN",
	UNKNOWN = "UNKNOWN",
	WARLOCK = "WARLOCK",
	WARRIOR = "WARRIOR",
}

local frames = {}

Basics.formatList = {}
for k in pairs(staticStyles) do
	table.insert(Basics.formatList, k)
	table.sort(Basics.formatList)
end

Basics.unitMap = unitMap

Basics.consoleOptions = {}
Basics.defaultOptions = {} 
Basics.frameSettings = {}

function Basics:OnInitialize()
	self.db = core:AcquireDBNamespace(moduleName)

	core:RegisterDefaults(moduleName, 'profile', self.defaultOptions)
	
	for english, locale in pairs(creatureTypeMap) do
		icons[locale] = english
	end
	creatureTypeMap = nil
	
	local args = self.consoleOptions
	
	local settings = self.frameSettings

	self.frames = frames
	for unit, unitSettings in pairs(settings) do
		local frame = CreateFrame("Frame", nil, unitSettings.parent)
		frame:SetFrameLevel(unitSettings.parent:GetFrameLevel()+2)
		frame.fontStrings = {}
		frames[unit] = frame
		-- Set up the FontStrings
		for text, textSettings in pairs(unitSettings.texts) do
			local mappedUnit = unitMap[unit] or unit
			local fontString = frame:CreateFontString(nil, "OVERLAY", (self.db.profile[mappedUnit] and self.db.profile[mappedUnit].font) or DEFAULT_FONT )
			fsParent[fontString] = textSettings.point[2]
			fontString:SetPoint(unpack(textSettings.point))
			frame.fontStrings[text] = fontString
		end
		if unitSettings.classIcon then
			local texture = frame:CreateTexture("OVERLAY")
			texture:SetHeight(unitSettings.classIcon.size)
			texture:SetWidth(unitSettings.classIcon.size)
			texture:SetPoint(unpack(unitSettings.classIcon.point))
			frame.classIcon = texture
		end
	end

	for unit, options in pairs(args) do
		core.consoleOptions.args[unit] = options
		if unit ~= "style" then
			options.args.font = {
				order = 1000,
				type = 'text',
				name = 'Font',
				desc = 'Set the font for this frame.',
				get = function() return self.db.profile[unit].font or DEFAULT_FONT end,
				set = function(v)
					self.db.profile[unit].font = v
					self:RefreshFont(unit)
				end,
				validate = fonts
			}
		end
	end
	self.consoleOptions = nil
	self.defaultOptions = nil
	self.frameSettings = nil
end

function Basics:OnEnable()
	self:RegisterEvent("UNIT_ENTERED_VEHICLE", "VehicleSwitch")
	self:RegisterEvent("UNIT_EXITED_VEHICLE", "VehicleSwitch")
	self:RefreshFontStrings()
	self:UpdateClassIcon("player")
end

function Basics:OnDisable()
	self:UnregisterAllEvents()
	for _, unitFrame in pairs(frames) do
		if unitFrame.classIcon then
			unitFrame.classIcon:Hide()
		end
		if unitFrame.fontStrings then
			for _, fontString in pairs(unitFrame.fontStrings) do
				LibStub("LibDogTag-3.0"):RemoveFontString(fontString)
			end
		end
	end
end

function Basics:VehicleSwitch(unit)
	-- Drop out if the unit is not player/party[1-4]
	if unit ~= "player" and not unit:match("^party[1-4]") then return end
	
	-- Drop out if we are in a vehicle and not exiting
	if inVehicle[unit] and UnitHasVehicleUI(unit) then return end
	
	-- Work out the name of the unit's pet
	local pet = "pet"
	if unit:match("^party[1-4]") then
		pet = unit:gsub("party", "partypet")
	end
	
	if UnitHasVehicleUI(unit) and not inVehicle[unit] and frames[unit] and frames[pet] then
		-- Has VehicleUI now but did not previously
		inVehicle[unit] = true
		frames[unit].fontStrings, frames[pet].fontStrings = frames[pet].fontStrings, frames[unit].fontStrings
		unitMap[unit], unitMap[pet] = unitMap[pet] or pet, unitMap[unit] or unit
	elseif inVehicle[unit] and not UnitHasVehicleUI(unit) and frames[unit] and frames[pet] then
		-- Had VehicleUI previously but does not now
		inVehicle[unit] = nil
		frames[unit].fontStrings, frames[pet].fontStrings = frames[pet].fontStrings, frames[unit].fontStrings
		unitMap[unit], unitMap[pet] = unitMap[pet] or pet, unitMap[unit] or unit
	end
	self:RefreshFontStrings()
end

function Basics:RefreshFontStrings(unit)
	if unit then
		local mappedUnit = unitMap[unit] or unit
		if mappedUnit and frames[unit] then
			for text, fontString in pairs(frames[unit].fontStrings) do
				local style = self.db.profile[mappedUnit][text]
				local dtString = staticStyles[style] or self.db.profile.style[style] or "ERROR"
				LibStub("LibDogTag-3.0"):AddFontString(fontString, fsParent[fontString] or fontString:GetParent(), dtString, "Unit", {["unit"] = unit})
			end
		end
	else
		for unit in pairs(frames) do
			self:RefreshFontStrings(unit)
		end
	end
end

function Basics:RefreshFont(unit)
	for frameUnit in pairs(frames) do
		if frameUnit:find('^'..unit) then
			for text, fontString in pairs(frames[frameUnit].fontStrings) do
				local font = self.db.profile[unit].font or DEFAULT_FONT
				fontString:SetFontObject(font)
			end
		end
	end
end

function Basics:UpdateClassIcon(unit)
	local classIcon = frames[unit] and frames[unit].classIcon
	local mappedUnit = unitMap[unit] or unit
	if classIcon and mappedUnit and UnitExists(unit) then
		if self.db.profile[mappedUnit].class then
			local icon = nil
			if UnitIsPlayer(unit) then
				_, icon = UnitClass(unit)
			else
				icon = UnitCreatureType(unit)
			end
			icon = (icon and icons[icon]) or "UNKNOWN"
			classIcon:SetTexture("Interface\\AddOns\\SimpleUnitFrames\\Icons\\" .. icon .. ".tga")
			classIcon:Show()
		else
			classIcon:Hide()
		end
	end
end
